package com.luojigou.product

import android.content.Intent
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.luojigou.product.base.BaseActivity
import com.luojigou.product.xiaodou.XDIntroduceActivity
import com.google.android.material.R.drawable as MaterialDrawable

class ProductSelectActivity : BaseActivity() {
    private lateinit var productAdapter: ProductAdapter
    private lateinit var productLayoutManager: GridLayoutManager
    private lateinit var productRecyclerView: RecyclerView

    private val productList = listOf(
        ProductModel(0, "小逗AI学习机", MaterialDrawable.ic_clock_black_24dp),
        ProductModel(1, "中逗AI学习机", MaterialDrawable.ic_keyboard_black_24dp),
        ProductModel(2, "大逗AI学习机", MaterialDrawable.material_ic_calendar_black_24dp),
    )

    override fun onCreate(savedInstanceState: android.os.Bundle?) {
        super.onCreate(savedInstanceState)
        setTitle("支持的设备列表")

        productRecyclerView = findViewById(R.id.rv_product_list)

        productAdapter = ProductAdapter(productList)
        productAdapter.setOnItemClickListener { p ->
            when (p.id) {
                0 -> startActivity(Intent(this, XDIntroduceActivity::class.java))
                1 -> { // TODO: 跳转到中逗AI学习机
                }

                2 -> { // TODO: 跳转到大逗AI学习机
                }
            }
        }
        productLayoutManager = GridLayoutManager(this, 2)
        productRecyclerView.layoutManager = productLayoutManager
        productRecyclerView.adapter = productAdapter
    }

    override fun getLayoutId(): Int = R.layout.activity_product_select

    class ProductAdapter() : RecyclerView.Adapter<ViewHolder>() {
        private val dataList = ArrayList<ProductModel>()

        private var onItemClickListener: ((ProductModel) -> Unit)? = null

        constructor(list: List<ProductModel>) : this() {
            dataList.addAll(list)
        }

        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
            val itemView = LayoutInflater.from(parent.context)
                .inflate(R.layout.item_product, parent, false)
            return ViewHolder(itemView)
        }

        override fun getItemCount(): Int = dataList.size

        override fun onBindViewHolder(holder: ViewHolder, position: Int) {
            val product = dataList[position]
            holder.name.text = product.name
            holder.img.setImageResource(product.icon)

            holder.itemView.setOnClickListener {
                onItemClickListener?.invoke(product)
            }
        }

        fun setOnItemClickListener(listener: (ProductModel) -> Unit) {
            onItemClickListener = listener
        }
    }

    class ViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        val name = itemView.findViewById<android.widget.TextView>(R.id.tv_product_name)
        val img = itemView.findViewById<android.widget.ImageView>(R.id.iv_product_img)
    }

    data class ProductModel(
        val id: Int,
        val name: String,
        val icon: Int,
    )
}