package com.luojigou.product.base

import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.graphics.drawable.ColorDrawable
import android.os.Bundle
import androidx.annotation.LayoutRes
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.ContextCompat
import com.luojigou.product.xiaodou.XDConnectActivity
import com.luojigou.product.xiaodou.XDConnectService

abstract class BaseActivity : AppCompatActivity() {
    private var _isFinishing = false
    private val finishReceiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent?) {
            finish()
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(getLayoutId())
        setupFinishReceiver()
    }

    @LayoutRes
    protected abstract fun getLayoutId(): Int

    protected fun setTitle(title: String) {
        supportActionBar?.run {
            this.title = title
            this.setDisplayHomeAsUpEnabled(true)
            this.setBackgroundDrawable(ColorDrawable(0x00000000))
        }
    }

    private fun setupFinishReceiver() {
        if (!_isFinishing && this !is XDConnectActivity) {
            val intentFilter = IntentFilter()
            intentFilter.addAction(XDConnectService.ACTION_CONNECT_FINISH)
            ContextCompat.registerReceiver(
                this,
                finishReceiver,
                intentFilter,
                ContextCompat.RECEIVER_EXPORTED
            )
            _isFinishing = true
        }
    }

    private fun teardownFinishReceiver() {
        if (_isFinishing) {
            unregisterReceiver(finishReceiver)
        }
    }

    override fun onDestroy() {
        teardownFinishReceiver()
        super.onDestroy()
    }
}